/*
 * File:   MAX6966.c
 * Author: peterharrison
 *
 * Created on 10 March 2012, 17:52
 *
 * The MAX6966/MAX6967 serial-interfaced peripherals provide microprocessors with 10 I/O ports rated to 7V.
 * Each port can be individually configured as either:
 * • A 20mA constant-current LED driver (static or pulse- width modulated (PWM)).
 * • A 10mA constant-current LED driver (static or PWM).
 * • An open-drain logic output.
 * • An overvoltage-protected Schmitt logic input.
 *
 * Analog and switching LED intensity control is built in:
 * • Individual 8-bit PWM control per output.
 * • Individual 1-bit analog control (half/full) per output.
 * • Global 3-bit analog control applies to all LED outputs.
 *
 * Find out more here:
 *  http://www.maxim-ic.com/datasheet/index.mvp/id/4539
 */
#include "stm32f4xx.h"
#include "MAX6966.h"
#include "hardwareSPI.h"

static uint8_t LEDbrightness[10];
static uint8_t currentLED;

void setLED(uint8_t led, uint8_t value) {
  if (led > 9) return;
  if (value < 3) value = 3;
  if (value > 250) value = 250;
  LEDbrightness[led] = value;
}

void MAX6966_putWord(uint16_t data) {
  MAX6966_SELECT();
  spiPutWord(data);
  MAX6966_DESELECT();
}

void MAX6966_update(void) {
  currentLED++;
  if (currentLED > 9) {
    currentLED = 0;
  }
  MAX6966_putWord((uint16_t) currentLED * 256 + LEDbrightness[currentLED]);
}

void MAX6966_init(void) {
  GPIO_InitTypeDef GPIO_InitStructure;
  int i;
  if (!spiConfigured) {
    HardwareSPI_init();
  }
  RCC_AHB1PeriphClockCmd(MAX6966_CS_GPIO_CLK, ENABLE);
  // now configure the pins themselves
  // they are all going to be fast push-pull outputs
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = MAX6966_CS_PIN;
  GPIO_Init(MAX6966_CS_GPIO_PORT, &GPIO_InitStructure);
  MAX6966_putWord(0x1005); // place device in run mode
  MAX6966_putWord(0x0AFF); // all ports as high impedance - over-written by intensity next
  for (i = 0; i < 10; i++) {
    setLED(i, 0); // set each port to active LED but off
  }
  currentLED = 0;
};

